@@ -1,4 +1,4 @@
-/* $Id: tinyproxy.c,v 1.46 2003/03/17 04:24:19 rjkaes Exp $
+/* $Id: tinyproxy.c,v 1.46 2003-03-17 04:24:19 rjkaes Exp $
  *
  * The initialize routine. Basically sets up all the initial stuff (logfile,
  * listening socket, config options, etc.) and then sits there and loops
@@ -165,6 +165,11 @@ main(int argc, char **argv)
 	}
 #endif				/* HAVE_SETRLIMIT */
 
+        /* Only allow u+rw bits. This may be required for some versions
+         * of glibc so that mkstemp() doesn't make us vulnerable.
+         */
+        umask(0177);
+
 	/* Default configuration file location */
 	config.config_file = DEFAULT_CONF_FILE;
 
